--
-- MI2_Config.lua
--
-- Configuration dialog related module of the MobInfo2 AddOn
--


-----------------------------------------------------------------------------
-- MI2_OptionsFrameOnLoad()
--
-- This "OnLoad" event handler for the options dialog installs the
-- ESC key handler.
-----------------------------------------------------------------------------
function MI2_OptionsFrameOnLoad()
	tinsert(UISpecialFrames, "frmMIConfig"); -- Esc Closes Options Frame
	UIPanelWindows["frmMIConfig"] = {area = "center", pushable = 0};

	PanelTemplates_SetNumTabs( MI2_MainOptionsFrame, 4 );
	MI2_MainOptionsFrame.selectedTab = 1;
	PanelTemplates_UpdateTabs( MI2_MainOptionsFrame );
	
	MI2_OptDisableMobInfoText:SetTextColor( 1.0, 1.0, 1.0 );

	-- mark all options with values instead of booleans
	MI2_OPTIONS["MI2_OptHealthPosX"].v=1;
	MI2_OPTIONS["MI2_OptHealthPosY"].v=1;
	MI2_OPTIONS["MI2_OptTargetFontSize"].v=1;
	MI2_OPTIONS["MI2_OptManaPosX"].v=1;
	MI2_OPTIONS["MI2_OptManaPosY"].v=1;
	MI2_OPTIONS["MI2_OptTargetFontSize"].v=1;

	MI2_OptHealthPosX:SetMinMaxValues( -50, 50 );
	MI2_OptHealthPosY:SetMinMaxValues( -40, 80 );
	MI2_OptManaPosX:SetMinMaxValues( -50, 50 );
	MI2_OptManaPosY:SetMinMaxValues( -40, 80 );
	MI2_OptTargetFontSize:SetMinMaxValues( 6, 20 );
end  -- MI2_OptionsFrameOnLoad()


-----------------------------------------------------------------------------
-- MI2_SetHealthOptionsState()
--
-- Set the state of health options in health options dialog to either
-- enabled or disabled, as indicated by "MobInfoConfig.DisableHealth".
-----------------------------------------------------------------------------
function MI2_SetHealthOptionsState()
	local r,g,b;

	if  MobInfoConfig.DisableHealth == 2  then
		MI2_MobHealthFrame:Hide();
	end
end  -- of MI2_SetHealthOptionsState()


-----------------------------------------------------------------------------
-- MI2_UpdateOptions()
--
-- Update state of all options in options dialog with correct values from
-- data structure "MobInfoConfig".
-----------------------------------------------------------------------------
function MI2_UpdateOptions()
	for index, value in MI2_OPTIONS do
		local option = string.sub( index, 8 );
		--DEFAULT_CHAT_FRAME:AddMessage( "MI_DBG: index="..index..", cmnd="..value.cmnd..", opt="..option );
		if  MobInfoConfig[option]  then
			if  value.v  then
				getglobal(index):SetValue( MobInfoConfig[option] );
			else
				getglobal(index):SetChecked( MobInfoConfig[option] );
			end
		end
	end
end  -- MI2_UpdateOptions()


-----------------------------------------------------------------------------
-- MI2_ShowOptionHelpTooltip()
--
-- Show help text for current hovered option in options dialog
-- in the game tooltip window.
-----------------------------------------------------------------------------
function MI2_ShowOptionHelpTooltip()
  GameTooltip_SetDefaultAnchor( GameTooltip, UIParent );
  GameTooltip:SetText( mifontWhite..MI2_OPTIONS[this:GetName()].text );
  
  GameTooltip:AddLine(mifontGold..MI2_OPTIONS[this:GetName()].help);
  if MI2_OPTIONS[this:GetName()].info then
    GameTooltip:AddLine(mifontGold..MI2_OPTIONS[this:GetName()].info);
  end
  GameTooltip:Show();
end -- of MI2_ShowOptionHelpTooltip()


-----------------------------------------------------------------------------
-- MI2_OptionsFrameOnShow()
--
-- Show help text for current hovered option in options dialog
-- in the game tooltip window.
-----------------------------------------------------------------------------
function MI2_OptionsFrameOnShow()
	txtMIConfigTitle:SetText( MI_TXT_CONFIG_TITLE );
	chattext(txtMIConfigTitle:GetText());
	MI2_UpdateOptions();
end  -- MI2_OptionsFrameOnShow()


function miConfig_OnMouseDown(arg1)
	if (arg1 == "LeftButton") then
		frmMIConfig:StartMoving();
	end
end


function miConfig_OnMouseUp(arg1)
	if (arg1 == "LeftButton") then
		frmMIConfig:StopMovingOrSizing();
	end
end


function miConfig_btnMIDone_OnClick()
	HideUIPanel(frmMIConfig);
	if MYADDONS_ACTIVE_OPTIONSFRAME then
		if (MYADDONS_ACTIVE_OPTIONSFRAME == this) then
			ShowUIPanel(myAddOnsFrame);
		end
	end
end


-----------------------------------------------------------------------------
-- MI2_TabButton_OnClick()
--
-- Event handler: one of the options dialog TABs has been clicked.
-- Show the corresponding options frame and hide all other option frames.
-----------------------------------------------------------------------------
function MI2_TabButton_OnClick( )
	PanelTemplates_Tab_OnClick( MI2_MainOptionsFrame );
	local selected = MI2_MainOptionsFrame.selectedTab;
	  
	-- choose special information frame if mob health has been disabled
	local healthFrame = MI2_FrmHealthOptions;
	if  MobInfoConfig.DisableHealth ~= 0  then
		healthFrame = MI2_FrmHealthDisabledInfo;
	end

	if  selected == 1  then
		MI2_FrmTooltipOptions:Show();
	else
		MI2_FrmTooltipOptions:Hide();
	end

	if  selected == 2  then
		healthFrame:Show();
	else
		healthFrame:Hide();
	end

	if  selected == 3  then
		MI2_FrmDatabaseOptions:Show();
	else
		MI2_FrmDatabaseOptions:Hide();
	end

	if  selected == 4  then
		MI2_FrmSearchOptions:Show();
	else
		MI2_FrmSearchOptions:Hide();
	end
end


-----------------------------------------------------------------------------
-- MI2_FontSelect_OnClick()
--
-- Event handler: one of the choices in the font selection box has been
-- clicked. Store it as a config option.
-----------------------------------------------------------------------------
function MI2_FontSelect_OnClick()
	local oldID = UIDropDownMenu_GetSelectedID( MI2_FontSelect );
	UIDropDownMenu_SetSelectedID( MI2_FontSelect, this:GetID());
	if  oldID ~= this:GetID()  then
		MobInfoConfig.TargetFont = this:GetID();
		MI2_MobHealth_SetPos();
	end
end  -- MI2_FontSelect_OnClick()


-----------------------------------------------------------------------------
-- MI2_FontSelect_Initialize()
--
-- Event handler: the font selection control is being loaded
-- Initialize the dropdown list with font names.
-----------------------------------------------------------------------------
function MI2_FontSelect_Initialize()
	local info;
	info = { text = "NumberFont"; value = 1; func = MI2_FontSelect_OnClick; };
	UIDropDownMenu_AddButton( info );
	info = { text = "GameFont"; value = 2; func = MI2_FontSelect_OnClick; };
	UIDropDownMenu_AddButton( info );
	info = { text = "ItemTextFont"; value = 3; func = MI2_FontSelect_OnClick; };
	UIDropDownMenu_AddButton( info );
end  -- MI2_FontSelect_Initialize()


-----------------------------------------------------------------------------
-- MI2_FontSelect_OnShow()
--
-- Event handler: font selection list is being shown.
-- Ensure that current selected font is shown correctly.
-----------------------------------------------------------------------------
function MI2_FontSelect_OnShow()
	UIDropDownMenu_Initialize( MI2_FontSelect, MI2_FontSelect_Initialize );
	UIDropDownMenu_SetSelectedID( MI2_FontSelect, MobInfoConfig.TargetFont );
end  -- MI2_FontSelect_OnShow()


-----------------------------------------------------------------------------
-- MI2_FontSelect_OnLoad()
--
-- Event handler: the font selection control is being loaded
-- Initialize the dropdown list for font names.
-----------------------------------------------------------------------------
function MI2_FontSelect_OnLoad()
	UIDropDownMenu_SetWidth( 100 );
	MI2_FontSelectTitle:SetText( MI2_OPTIONS["MI2_FontSelect"].text );
end  -- MI2_FontSelect_OnLoad()


-----------------------------------------------------------------------------
-- MI2_DbOptionsFrameOnShow()
--
-----------------------------------------------------------------------------
function MI2_DbOptionsFrameOnShow()
	local mobDbSize = 0;
	local healthDbSize = 0;
	local playerDbSize = 0;

	-- count and diplay size of MobInfo database
	for index, value in MobInfoDB do
		mobDbSize = mobDbSize + 1;
	end
	MI2_TxtMobDbSize:SetText( MI_TXT_MOB_DB_SIZE..mifontWhite..mobDbSize );

	-- count and diplay size of MobHealth database
	for index, value in MobHealthDB do
		healthDbSize = healthDbSize + 1;
	end
	MI2_TxtHealthDbSize:SetText( MI_TXT_HEALTH_DB_SIZE..mifontWhite..healthDbSize );

	-- count and diplay size of MobHealthPlayer database
	for index, value in MobHealthPlayerDB do
		playerDbSize = playerDbSize + 1;
	end
	MI2_TxtPlayerDbSize:SetText( MI_TXT_PLAYER_DB_SIZE..mifontWhite..playerDbSize );

	-- update mob index display and state of "clear mob" button
	MI2_TxtTargetIndex:SetText( MI_TXT_CUR_TARGET..mifontWhite..(MI2_Target.mobIndex or "---") );
	local healthIndex = (MI2_Target.name or "---")..":"..(MI2_Target.level or 0);
	if MI2_Target.mobIndex and MobInfoDB[MI2_Target.mobIndex] then
		MI2_OptClearTarget:Enable();
	else
		MI2_OptClearTarget:Disable();
	end
end  -- MI2_DbOptionsFrameOnShow()
